
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

yarip.Loader = function(overlay)
{
	this.overlay = overlay;
	this.found = false;

	this.load = function(doc, pages)
	{
		if (!doc || doc.nodeName != "#document" || !pages) return false;

		var address = yarip.getAddress(doc, yarip.MODES.page);
		if (!address) return false;

		var aAddress = yarip.getAddressArray(doc, pages);
		if (this.doWhitelisting(doc, address, aAddress, pages)) this.doBlacklisting(doc, address, aAddress, pages);

		var found = this.found;
		this.found = false;
		return found;
	}

	this.doWhitelisting = function(doc, pageAddress, aAddress, pages)
	{
		var hasNoBlacklist = true;

		for (var i = aAddress.length - 1; i >= 0; i--)
		{
			var whitelist = pages[aAddress[i]].whitelist;

			for (var xValue in whitelist.xpaths)
			{
				var xpath = whitelist.xpaths[xValue];

				if (!xpath || !xpath.deep && this.equalAddresses(pageAddress, aAddress[i])) continue;

				this.overlay.changesMade = true;

				var elements = yarip.getElementsByXPath(doc, xValue);
				if (elements) whitelist.xpaths[xValue].found++; // increment `found'
				else
				{
					whitelist.xpaths[xValue].notFound++; // increment `not found'
					continue;
				}

				this.found = true;
				hasNoBlacklist = hasNoBlacklist && pages[aAddress[i]].blacklist.length == 0;

				for (var j = 0; j < elements.snapshotLength; j++)
				{
					var element = elements.snapshotItem(j);
					while (element && element.nodeType == 1 && element.getAttribute("status") != "whitelisted")
					{
						element.setAttribute("status", "whitelisted");
						element = element.parentNode;
					}
				}
			}
		}

		if (this.found && hasNoBlacklist)
		{
			yarip.blacklist(doc, pages, pageAddress, new yarip.XPath("//*[not(@status='whitelisted')]")); // remove all except whitelisted
			var elements = yarip.getElementsByXPath(doc, "//*[@status='whitelisted']");
			if (!elements) return false;

			var obj = {}
			for (var i in doc.body.attributes) if (doc.body.attributes[i] && doc.body.attributes[i].name != "status") obj[doc.body.attributes[i].name] = true;
			for (var n in obj) doc.body.removeAttribute(n);
//			doc.body.style.margin = "0";

			for (var i = 0; i < elements.snapshotLength; i++)
			{
				var element = elements.snapshotItem(i);
				element.style.background = null;
				if (!element.attributes) continue;

				if (!element.localName.match(/div|table|td|th/i)) continue;

				for (var j in element.attributes)
				{
					if (!element.attributes[j]) continue;

					var n = element.attributes[j].name;
					if (!n) continue;

					switch (n.toLowerCase())
					{
						case "align":
							element.setAttribute(n, "center");
							break;

						case "border":
						case "cellpadding":
						case "cellspacing":
							element.setAttribute(n, "0");
							break;

						case "background":
						case "bgcolor":
						case "colspan":
						case "height":
						case "rowspan":
						case "style":
							element.removeAttribute(n);
							break;

						case "valign":
							element.setAttribute(n, "top");
					}
				}
			}

			return false;
		}

		return true;
	}

	this.doBlacklisting = function(doc, pageAddress, aAddress, pages)
	{
		for (var i = aAddress.length - 1; i >= 0; i--)
		{
			var blacklist = pages[aAddress[i]].blacklist;

			for (var xValue in blacklist.xpaths)
			{
				var xpath = blacklist.xpaths[xValue];

				if (!xpath || !xpath.deep && this.equalAddresses(pageAddress, aAddress[i])) continue;

				this.overlay.changesMade = true;

				if (yarip.blacklist(doc, pages, aAddress[i], xpath))
				{
					this.found = true;
					blacklist.xpaths[xValue].found++; // increment `found'
				}
				else blacklist.xpaths[xValue].notFound++; // increment `not found'
			}
		}
	}

	this.equalAddresses = function(pageAddress, address)
	{
		pageAddress = pageAddress.replace(/[/]+$/, "");
		address = address.replace(/[/]+$/, "");
		if (address.match(yarip.noSchemeRegExp)) pageAddress = pageAddress.replace(/^[^:/?#]+:[/]{2}/, ""); // remove scheme
		return pageAddress != address;
	}
}
